package de.mospace.apps.metronome;

/* Metronome
 * Copyright (C) 2007 Moritz Ringler
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.sound.midi.Instrument;
import javax.sound.midi.MidiUnavailableException;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.ListCellRenderer;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class MetronomeGUI extends JFrame{
    final private MidiMetronome metronome = new MidiMetronome();
    private int bpm = 60;
    final private JLabel temponame = new JLabel("Larghetto");
    final private SortedMap<Integer, String> tempi = new TreeMap<Integer, String>();
    final private static int MAX_BPM = 300;
    final private JSlider bpmSlider = new JSlider(JSlider.VERTICAL, 0, MAX_BPM, bpm);
    final private JSpinner bpmSpinner = new JSpinner(new SpinnerNumberModel(bpm, 0, MAX_BPM, bpm));
    final private JButton startStop = new JButton("Start");

    public MetronomeGUI() throws MidiUnavailableException{
        super("Metronome");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        ChangeListener bpmCL = new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                int newbpm = -1;
                Object source = e.getSource();
                if(source instanceof JSpinner){
                    newbpm = (
                        (Integer) ((JSpinner) source).getValue()
                        ).intValue();
                } else if(source instanceof JSlider){
                    newbpm = (int) ((JSlider) source).getValue();
                }
                setTempo(newbpm);
            }
        };
        bpmSlider.addChangeListener(bpmCL);
        bpmSlider.setMajorTickSpacing(60);
        bpmSlider.setPaintTicks(true);
        bpmSlider.setPaintTrack(true);
        bpmSlider.setPaintLabels(true);

        bpmSpinner.addChangeListener(bpmCL);
        defaultTempi();
        startStop.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                toggleOnOff();
            }
        });

        final JComboBox clix = new JComboBox(metronome.getAvailableInstruments());
        final ListCellRenderer defaultRenderer = clix.getRenderer();
        final ListCellRenderer renderer = new ListCellRenderer(){
            public Component getListCellRendererComponent(JList list,
                    Object value, int index, boolean isSelected,
                    boolean cellHasFocus){
                return defaultRenderer.getListCellRendererComponent(
                        list, (value == null)
                            ? null
                            : ((Instrument) value).getName(),
                            index, isSelected, cellHasFocus);
            }
        };
        clix.setRenderer(renderer);
        if(metronome.getInstrument() != null){
            clix.setSelectedItem(metronome.getInstrument());
        }
        clix.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                JComboBox jcb = (JComboBox) e.getSource();
                Instrument clik =
                        (Instrument) jcb.getSelectedItem();
                if(clik != null){
                    try{
                        metronome.setInstrument(clik);
                    } catch (RuntimeException ex){
                        ex.printStackTrace();
                        JOptionPane.showMessageDialog(MetronomeGUI.this,
                            ex, "Error", JOptionPane.WARNING_MESSAGE);
                    }  catch (Exception ex){
                        JOptionPane.showMessageDialog(MetronomeGUI.this,
                            ex, "Error", JOptionPane.WARNING_MESSAGE);
                    }
                }
            }
        });
        Box contentPane = Box.createVerticalBox();
        contentPane.setOpaque(true);
        setContentPane(contentPane);
        add(temponame);
        add(bpmSpinner);
        add(bpmSlider);
        add(clix);
        add(startStop);
        setTempo(60);
    }

    public JComponent add(JComponent comp){
        comp.setAlignmentX(Component.CENTER_ALIGNMENT);
        getContentPane().add(comp);
        return comp;
    }

    private void defaultTempi(){
        tempi.put(1, "Grave");
        tempi.put(40, "Largo");
        tempi.put(60, "Larghetto");
        tempi.put(66, "Adagio");
        tempi.put(76, "Andante");
        tempi.put(108, "Moderato");
        tempi.put(120, "Allegro");
        tempi.put(168, "Presto");
        tempi.put(208, "Prestissimo");
    }

    private void toggleOnOff(){
        if(metronome.isRunning()){
            metronome.stop();
            startStop.setText("Start");
        } else {
            if(bpm == 0){
                setTempo(60);
            }
            try{
                metronome.start(bpm);
                startStop.setText("Stop");
            } catch (Exception ex){
                JOptionPane.showMessageDialog(MetronomeGUI.this,
                        ex, "Error", JOptionPane.WARNING_MESSAGE);
            }
        }
    }

    private void setTempo(int bpm){
        if (bpmSlider.getValue() != bpm){
            bpmSlider.setValue(bpm);
        }
        if (!bpmSpinner.getValue().equals(bpm)){
            bpmSpinner.setValue(bpm);
        }
        if(bpm <= 0){
            if(!bpmSlider.getValueIsAdjusting() && metronome.isRunning()){
                toggleOnOff();
            }
            return;
        }
        if(this.bpm != bpm){
            this.bpm = bpm;
            Integer key = 1;
            for(Integer ikey : tempi.keySet()){
                if(ikey > bpm){
                    break;
                } else {
                    key = ikey;
                }
            }
            temponame.setText(tempi.get(key));
            metronome.setTempo(bpm);
            repaint();
        }
    }

}
